@@ -38,6 +38,8 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <getopt.h>
+#include <time.h>
 
 extern "C" {
 #include "selfpipe.h"
@@ -62,26 +64,137 @@ void serve( int host_fd,
 	    Terminal::Complete &terminal,
 	    ServerConnection &network );
 
-int run_server( const char *desired_ip, const char *desired_port );
+int run_server( const char *desired_ip, const char *desired_port,
+		char *command[], const int colors );
 
 using namespace std;
 
+void print_usage( const char *argv0 )
+{
+  fprintf( stderr, "Usage: %s new [-s] [-i LOCALADDR] [-p PORT] [-c COLORS] [-- COMMAND...]\n", argv0 );
+}
+
+/* Simple spinloop */
+void spin( void )
+{
+  static unsigned int spincount = 0;
+  spincount++;
+
+  if ( spincount > 10 ) {
+    struct timespec req;
+    req.tv_sec = 0;
+    req.tv_nsec = 100000000; /* 0.1 sec */
+    nanosleep( &req, NULL );
+  }
+}
+
+string get_SSH_IP( void )
+{
+  const char *SSH_CONNECTION = getenv( "SSH_CONNECTION" );
+  fatal_assert( SSH_CONNECTION );
+  char *SSH_writable = strdup( SSH_CONNECTION );
+  fatal_assert( SSH_writable );
+  strtok( SSH_writable, " " );
+  strtok( NULL, " " );
+  const char *local_interface_IP = strtok( NULL, " " );
+  fatal_assert( local_interface_IP );
+  return string( local_interface_IP );
+}
+
 int main( int argc, char *argv[] )
 {
+  /* For security, make sure we don't dump core */
+  Crypto::disable_dumping_core();
+
   char *desired_ip = NULL;
   char *desired_port = NULL;
-  if ( argc == 1 ) {
-    desired_ip = NULL;
+  char **command = NULL;
+  int colors = 0;
+
+  /* strip off command */
+  for ( int i = 0; i < argc; i++ ) {
+    if ( 0 == strcmp( argv[ i ], "--" ) ) { /* -- is mandatory */
+      if ( i != argc - 1 ) {
+	command = argv + i + 1;
+      }
+      argc = i; /* rest of options before -- */
+      break;
+    }
+  }
+
+  /* Parse new command-line syntax */
+  if ( (argc >= 2)
+       && (strcmp( argv[ 1 ], "new" ) == 0) ) {
+    /* new option syntax */
+    int opt;
+    while ( (opt = getopt( argc, argv, "i:p:c:s" )) != -1 ) {
+      switch ( opt ) {
+      case 'i':
+	desired_ip = optarg;
+	break;
+      case 'p':
+	desired_port = optarg;
+	break;
+      case 's':
+	desired_ip = strdup( get_SSH_IP().c_str() );
+	fatal_assert( desired_ip );
+	break;
+      case 'c':
+	colors = myatoi( optarg );
+	break;
+      default:
+	print_usage( argv[ 0 ] );
+	/* don't die on unknown options */
+      }
+    }
+  } else if ( argc == 1 ) {
+    /* legacy argument parsing for older client wrapper script */
+    /* do nothing */
   } else if ( argc == 2 ) {
     desired_ip = argv[ 1 ];
   } else if ( argc == 3 ) {
     desired_ip = argv[ 1 ];
     desired_port = argv[ 2 ];
   } else {
-    fprintf( stderr, "Usage: %s [LOCALADDR] [PORT]\n", argv[ 0 ] );
+    print_usage( argv[ 0 ] );
     exit( 1 );
   }
 
+  /* Sanity-check arguments */
+  if ( desired_ip
+       && ( strspn( desired_ip, "0123456789." ) != strlen( desired_ip ) ) ) {
+    fprintf( stderr, "%s: Bad IP address (%s)\n", argv[ 0 ], desired_ip );
+    print_usage( argv[ 0 ] );
+    exit( 1 );
+  }
+
+  if ( desired_port
+       && ( strspn( desired_port, "0123456789" ) != strlen( desired_port ) ) ) {
+    fprintf( stderr, "%s: Bad UDP port (%s)\n", argv[ 0 ], desired_port );
+    print_usage( argv[ 0 ] );
+    exit( 1 );
+  }
+
+  /* Get shell */
+  char *my_argv[ 2 ];
+  if ( !command ) {
+    /* get shell name */
+    struct passwd *pw = getpwuid( geteuid() );
+    if ( pw == NULL ) {
+      perror( "getpwuid" );
+      exit( 1 );
+    }
+
+    string shell_name( pw->pw_shell );
+    if ( shell_name.empty() ) { /* empty shell means Bourne shell */
+      shell_name = "/bin/sh";
+    }
+
+    my_argv[ 0 ] = pw->pw_shell;
+    my_argv[ 1 ] = NULL;
+    command = my_argv;
+  }
+
   /* Adopt implementation locale */
   if ( NULL == setlocale( LC_ALL, "" ) ) {
     perror( "setlocale" );
@@ -95,7 +208,7 @@ int main( int argc, char *argv[] )
   }
 
   try {
-    return run_server( desired_ip, desired_port );
+    return run_server( desired_ip, desired_port, command, colors );
   } catch ( Network::NetworkException e ) {
     fprintf( stderr, "Network exception: %s: %s\n",
 	     e.function.c_str(), strerror( e.the_errno ) );
@@ -107,7 +220,8 @@ int main( int argc, char *argv[] )
   }
 }
 
-int run_server( const char *desired_ip, const char *desired_port ) {
+int run_server( const char *desired_ip, const char *desired_port,
+		char *command[], const int colors ) {
   /* get initial window size */
   struct winsize window_size;
   if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &window_size ) < 0 ) {
@@ -176,13 +290,18 @@ int run_server( const char *desired_ip, const char *desired_port ) {
   if ( child == 0 ) {
     /* child */
 
+    setsid(); /* may fail */
+
     /* unblock signals */
     sigset_t signals_to_block;
     fatal_assert( sigemptyset( &signals_to_block ) == 0 );
     fatal_assert( sigprocmask( SIG_SETMASK, &signals_to_block, NULL ) == 0 );
 
     /* set TERM */
-    if ( setenv( "TERM", "xterm", true ) < 0 ) {
+    const char default_term[] = "xterm";
+    const char color_term[] = "xterm-256color";
+
+    if ( setenv( "TERM", (colors == 256) ? color_term : default_term, true ) < 0 ) {
       perror( "setenv" );
       exit( 1 );
     }
@@ -199,23 +318,17 @@ int run_server( const char *desired_ip, const char *desired_port ) {
       exit( 1 );
     }
 
-    /* get shell name */
-    struct passwd *pw = getpwuid( geteuid() );
-    if ( pw == NULL ) {
-      perror( "getpwuid" );
-      exit( 1 );
-    }
+    /* prepend '-' to make login shell */
+    string executable( command[ 0 ] );
+    string argv0 = "-" + executable;
 
-    char *my_argv[ 2 ];
-    my_argv[ 0 ] = strdup( pw->pw_shell );
-    fatal_assert( my_argv[ 0 ] );
-    my_argv[ 1 ] = NULL;
+    command[ 0 ] = strdup( argv0.c_str() );
+    fatal_assert( command[ 0 ] );
     
-    if ( execv( pw->pw_shell, my_argv ) < 0 ) {
-      perror( "execve" );
-      exit( 1 );
+    if ( execvp( executable.c_str(), command ) < 0 ) {
+      perror( "execvp" );
+      _exit( 1 );
     }
-    exit( 0 );
   } else {
     /* parent */
 
@@ -279,10 +392,10 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 
   #ifdef HAVE_UTEMPTER
   bool connected_utmp = false;
-  #endif
 
   struct in_addr saved_addr;
   saved_addr.s_addr = 0;
+  #endif
 
   while ( 1 ) {
     try {
@@ -375,7 +488,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	ssize_t bytes_read = read( pollfds[ 1 ].fd, buf, buf_size );
 	if ( bytes_read == 0 ) { /* EOF */
 	  if ( !network.has_remote_addr() ) {
-	    return;
+	    spin(); /* let 60-second timer take care of this */
 	  } else if ( !network.shutdown_in_progress() ) {
 	    network.start_shutdown();
 	  }
@@ -426,7 +539,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	if ( network.has_remote_addr() ) {
 	  network.start_shutdown();
 	} else {
-	  break;
+	  spin(); /* let 60-second timer take care of this */
 	}
       }
 
@@ -477,7 +590,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
       network.tick();
     } catch ( Network::NetworkException e ) {
       fprintf( stderr, "%s: %s\n", e.function.c_str(), strerror( e.the_errno ) );
-      sleep( 1 );
+      spin();
     } catch ( Crypto::CryptoException e ) {
       fprintf( stderr, "Crypto exception: %s\n", e.text.c_str() );
     }
