@@ -21,6 +21,8 @@
 #include <algorithm>
 #include <list>
 #include <stdio.h>
+#include <stdlib.h>
+#include <time.h>
 
 #include "transportsender.h"
 #include "transportfragment.h"
@@ -44,7 +46,8 @@ TransportSender<MyState>::TransportSender( Connection *s_connection, MyState &in
     ack_num( 0 ),
     pending_data_ack( false ),
     SEND_MINDELAY( 15 ),
-    last_heard( 0 )
+    last_heard( 0 ),
+    prng()
 {
 }
 
@@ -257,6 +260,17 @@ void TransportSender<MyState>::rationalize_states( void )
   }
 }
 
+template <class MyState>
+const string TransportSender<MyState>::make_chaff( void )
+{
+  const size_t CHAFF_MAX = 16;
+  const size_t chaff_len = prng.uint8() % (CHAFF_MAX + 1);
+
+  char chaff[ CHAFF_MAX ];
+  prng.fill( chaff, chaff_len );
+  return string( chaff, chaff_len );
+}
+
 template <class MyState>
 void TransportSender<MyState>::send_in_fragments( string diff, uint64_t new_num )
 {
@@ -268,6 +282,7 @@ void TransportSender<MyState>::send_in_fragments( string diff, uint64_t new_num
   inst.set_ack_num( ack_num );
   inst.set_throwaway_num( sent_states.front().num );
   inst.set_diff( diff );
+  inst.set_chaff( make_chaff() );
 
   if ( new_num == uint64_t(-1) ) {
     shutdown_tries++;
