@@ -159,7 +159,7 @@ std::string Emulator::open( void )
 
 std::string Emulator::close( void )
 {
-  return std::string( "\033[?1l\033[!p" );
+  return std::string( "\033[?1l\033[0m" );
 }
 
 void Emulator::resize( size_t s_width, size_t s_height )
