@@ -29,13 +29,13 @@ namespace Terminal {
     std::string str;
 
     int cursor_x, cursor_y;
-    std::string current_rendition_string;
+    Renditions current_rendition;
 
     Framebuffer last_frame;
 
     FrameState( const Framebuffer &s_last )
       : x(0), y(0),
-	str(), cursor_x(0), cursor_y(0), current_rendition_string(),
+	str(), cursor_x(0), cursor_y(0), current_rendition( 0 ),
 	last_frame( s_last )
     {
       str.reserve( 1024 );
@@ -52,9 +52,15 @@ namespace Terminal {
     bool has_ech; /* erase character is part of vt200 but not supported by tmux
 		     (or by "screen" terminfo entry, which is what tmux advertises) */
 
+    bool has_bce; /* erases result in cell filled with background color */
+
+    int posterize_colors; /* downsample input colors >8 to [0..7] */
+
     void put_cell( bool initialized, FrameState &frame, const Framebuffer &f ) const;
 
   public:
+    void downgrade( Framebuffer &f ) const { if ( posterize_colors ) { f.posterize(); } }
+
     std::string new_frame( bool initialized, const Framebuffer &last, const Framebuffer &f ) const;
 
     Display( bool use_environment );
