@@ -320,6 +320,15 @@ void Framebuffer::soft_reset( void )
   ds.clear_saved_cursor();
 }
 
+void Framebuffer::posterize( void )
+{
+  for ( BOOST_AUTO( i, rows.begin() ); i != rows.end(); i++ ) {
+    for ( BOOST_AUTO( j, i->cells.begin() ); j != i->cells.end(); j++ ) {
+      j->renditions.posterize();
+    }
+  }
+}
+
 void Framebuffer::resize( int s_width, int s_height )
 {
   assert( s_width > 0 );
@@ -369,6 +378,7 @@ Renditions::Renditions( int s_background )
     background_color( s_background )
 {}
 
+/* This routine cannot be used to set a color beyond the 8-color set. */
 void Renditions::set_rendition( int num )
 {
   if ( num == 0 ) {
@@ -377,10 +387,18 @@ void Renditions::set_rendition( int num )
     return;
   }
 
-  if ( (30 <= num) && (num <= 39) ) { /* foreground color */
+  if ( num == 39 ) {
+    foreground_color = 0;
+    return;
+  } else if ( num == 49 ) {
+    background_color = 0;
+    return;
+  }
+
+  if ( (30 <= num) && (num <= 37) ) { /* foreground color in 8-color set */
     foreground_color = num;
     return;
-  } else if ( (40 <= num) && (num <= 49) ) { /* background color */
+  } else if ( (40 <= num) && (num <= 47) ) { /* background color in 8-color set */
     background_color = num;
     return;
   }
@@ -394,6 +412,20 @@ void Renditions::set_rendition( int num )
   }
 }
 
+void Renditions::set_foreground_color( int num )
+{
+  if ( (0 <= num) && (num <= 255) ) {
+    foreground_color = 30 + num;
+  }
+}
+
+void Renditions::set_background_color( int num )
+{
+  if ( (0 <= num) && (num <= 255) ) {
+    background_color = 40 + num;
+  }
+}
+
 std::string Renditions::sgr( void ) const
 {
   std::string ret;
@@ -404,21 +436,97 @@ std::string Renditions::sgr( void ) const
   if ( blink ) ret.append( ";5" );
   if ( inverse ) ret.append( ";7" );
   if ( invisible ) ret.append( ";8" );
-  if ( foreground_color ) {
+
+  if ( foreground_color
+       && (foreground_color <= 37) ) {
+    /* ANSI foreground color */
     char col[ 8 ];
     snprintf( col, 8, ";%d", foreground_color );
     ret.append( col );
   }
-  if ( background_color ) {
+
+  if ( background_color
+       && (background_color <= 47) ) {
     char col[ 8 ];
     snprintf( col, 8, ";%d", background_color );
     ret.append( col );
   }
+
   ret.append( "m" );
 
+  if ( foreground_color > 37 ) { /* use 256-color set */
+    char col[ 64 ];
+    snprintf( col, 64, "\033[38;5;%dm", foreground_color - 30 );
+    ret.append( col );
+  }
+
+  if ( background_color > 47 ) { /* use 256-color set */
+    char col[ 64 ];
+    snprintf( col, 64, "\033[48;5;%dm", background_color - 40 );
+    ret.append( col );
+  }
+
   return ret;
 }
 
+/* Reduce 256 "standard" colors to the 8 ANSI colors. */
+
+/* We could do something fancy like find the nearest system color in
+   the deltaE(2000) sense, but for "business graphics," the colorimetric
+   intent is probably less important than just having separate colors
+   be separate as much as possible. */
+
+/* Terminal emulators generally agree on the (R',G',B') values of the
+   "standard" 256-color pallette beyond #15, but for the first 16
+   colors there is disagreement. Most terminal emulators are roughly
+   self-consistent, except on Ubuntu's gnome-terminal where "ANSI
+   blue" (#4) has been replaced with the aubergine system-wide
+   color. See
+   https://lists.ubuntu.com/archives/ubuntu-devel/2011-March/032726.html
+
+   Terminal emulators that advertise "xterm" are inconsistent on the
+   handling of initc to change the contents of a cell in the color
+   pallette. On RIS (reset to initial state) or choosing reset from
+   the user interface, xterm resets all entries, but gnome-terminal
+   only resets entries beyond 16. (rxvt doesn't reset any entries,
+   and Terminal.app ignores initc.) On initc, xterm applies changes
+   immediately (but slowly), but gnome-terminal's changes are only
+   prospective unless the user resizes the terminal.
+
+   mosh ignores initc for now, despite advertising xterm-256color. */
+
+/* Table mapping common color cube for [16 .. 255]
+   to xterm's system colors (0 .. 7) with closest
+   CIE deltaE(2000). */
+static const char standard_posterization[] = {
+  0, 1, 2, 3, 4, 5, 6, 7, 7, 1, 2, 3, 4, 5, 6, 7,
+  0, 4, 4, 4, 4, 4, 0, 0, 4, 4, 4, 4, 2, 6, 6, 6,
+  6, 4, 2, 2, 6, 6, 6, 6, 2, 2, 2, 6, 6, 6, 2, 2,
+  2, 2, 6, 6, 1, 4, 4, 4, 4, 4, 0, 0, 4, 4, 4, 4,
+  2, 2, 6, 6, 4, 5, 2, 2, 6, 6, 6, 6, 2, 2, 2, 6,
+  6, 6, 2, 2, 2, 2, 6, 6, 1, 5, 5, 4, 4, 4, 1, 1,
+  5, 5, 5, 5, 3, 3, 7, 5, 5, 5, 2, 2, 2, 6, 7, 7,
+  2, 2, 2, 6, 6, 6, 2, 2, 2, 2, 6, 6, 1, 5, 5, 5,
+  5, 5, 1, 1, 5, 5, 5, 5, 3, 1, 7, 5, 5, 5, 3, 3,
+  3, 7, 7, 7, 3, 3, 2, 7, 6, 7, 3, 2, 2, 2, 6, 6,
+  1, 5, 5, 5, 5, 5, 1, 1, 5, 5, 5, 5, 3, 1, 1, 5,
+  5, 5, 3, 3, 7, 7, 7, 7, 3, 3, 3, 7, 7, 7, 3, 3,
+  3, 3, 7, 7, 1, 1, 5, 5, 5, 5, 1, 1, 5, 5, 5, 5,
+  1, 1, 1, 5, 5, 5, 3, 7, 7, 7, 7, 7, 3, 3, 3, 7,
+  7, 7, 3, 3, 3, 3, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 1, 1, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7 };
+
+void Renditions::posterize( void )
+{
+  if ( foreground_color ) {
+    foreground_color = 30 + standard_posterization[ foreground_color - 30 ];
+  }
+
+  if ( background_color ) {
+    background_color = 40 + standard_posterization[ background_color - 40 ];
+  }
+}
+
 void Row::reset( int background_color )
 {
   for ( std::vector<Cell>::iterator i = cells.begin();
